package com.gic.auth.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.auth.dao.mapper.TabSysMenuMapper;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.service.MenuService;
import com.gic.commons.util.EntityUtil;
import org.springframework.transaction.annotation.Transactional;

@Service("menuService")
public class MenuServiceImpl implements MenuService {
    @Autowired
    private TabSysMenuMapper tabSysMenuMapper;
    @Override
    public List<TabSysMenu> listMenu(MenuListQO params) {
        return this.tabSysMenuMapper.listMenu(params);
    }

    @Override
    public TabSysMenu getMenuById(Integer menuId) {
        return this.tabSysMenuMapper.selectByPrimaryKey(menuId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int savePage(MenuDTO menuDTO) {
        TabSysMenu record = EntityUtil.changeEntityNew(TabSysMenu.class, menuDTO);
        record.setCreateTime(new Date());
        record.setUpdateTime(new Date());
        record.setStatus(1);
        tabSysMenuMapper.insertSelective(record);
        return record.getMenuId();
    }

    @Override
    public int getMaxSortByParentId(Integer parentId,  String project) {
        return tabSysMenuMapper.getMaxSortByParentId(parentId, project);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updatePage(MenuDTO menuDTO) {
        TabSysMenu record = EntityUtil.changeEntityNew(TabSysMenu.class, menuDTO);
        record.setUpdateTime(new Date());
        tabSysMenuMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public List<TabSysMenu> listMenuItemByParentId(Integer parentId) {
        List<TabSysMenu> list = tabSysMenuMapper.listMenuItemByParentId(parentId);
        if (CollectionUtils.isNotEmpty(list)) {
            return list;
        }
        return new ArrayList<>();
    }

    @Override
    public void setSort(TabSysMenu record, Integer sortValue) {
        List<TabSysMenu> list = tabSysMenuMapper.listMenuByParentId(record.getParentId(), record.getProject());
        Integer fromSortValue = record.getSort();
        if (fromSortValue > sortValue) {
            //向上拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysMenu temp = list.get(i);
                //如果大于sortValue，都需要降低排序值,往后推
                boolean isNeedDown = temp.getSort() >= sortValue && temp.getSort() < fromSortValue;
                if (isNeedDown) {
                    updateSort(list.get(i + 1).getSort(), temp.getMenuId());
                }
            }
        } else if (fromSortValue < sortValue) {
            //向下拖拽
            for (int i = 0, length = list.size(); i < length; i++) {
                TabSysMenu temp = list.get(i);
                boolean isNeedUp = temp.getSort() <= sortValue && temp.getSort() > fromSortValue;
                if (isNeedUp) {
                    updateSort(list.get(i - 1).getSort(), temp.getMenuId());
                }
            }
        }
        updateSort(sortValue, record.getMenuId());
    }

    @Override
    public Page<TabSysMenu> pageMenuOperationItem(MenuOperationItemListQO params) {
        PageHelper.startPage(params.getCurrentPage(), params.getPageSize());
        List<TabSysMenu> list = tabSysMenuMapper.pageMenuOperationItem(params);
        return (Page<TabSysMenu>) list;
    }

    @Override
    public List<TabSysMenu> selectByIds(List<Integer> menuIdList) {
        if (CollectionUtils.isEmpty(menuIdList)) {
            return new ArrayList<>();
        }
        return tabSysMenuMapper.selectByIds(menuIdList);
    }

    @Override
    public List<TabSysMenu> selectByIdsAndOrderBy(List<Integer> menuIdList, String orderByStr) {
        if (CollectionUtils.isEmpty(menuIdList)) {
            return new ArrayList<>();
        }
        return tabSysMenuMapper.selectByIdsAndOrderBy(menuIdList, orderByStr);
    }

    @Override
    public List<AuthItemListDTO> listAuthItemByMenuId(Integer menuId) {
        return tabSysMenuMapper.listAuthItemByMenuId(menuId);
    }

    @Override
    public boolean isHasChild(Integer menuId) {
        int count = tabSysMenuMapper.countChildByParentId(menuId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public List<TabSysMenu> listMenuTree(String project, String versionCode) {
        return tabSysMenuMapper.listMenuTree(project, versionCode);
    }

    @Override
    public List<TabSysMenu> selectByProjectList(List<String> projectList) {
        return tabSysMenuMapper.selectByProjectList(projectList);
    }

    @Override
    public List<TabSysMenu> selectAppMenuByParam(List<String> projectList, List<String> versionList) {
        return tabSysMenuMapper.selectAppMenuByParam(projectList, versionList);
    }

    @Override
    public boolean isRepeatMenuCode(String code, Integer menuId) {
        int count = tabSysMenuMapper.countByMenuCode(code, menuId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public void updateParentCodeByParentId(Integer parentId, String parentCode) {
        tabSysMenuMapper.updateParentCodeByParentId(parentId, parentCode);
    }

    @Override
    public List<TabSysMenu> listAdminOnlyOperationItem() {
        return tabSysMenuMapper.listAdminOnlyOperationItem();
    }

    @Override
    public void deleteByMenuIdList(List<Integer> menuIdList) {
        if (CollectionUtils.isNotEmpty(menuIdList)) {
            tabSysMenuMapper.deleteByMenuIdList(menuIdList);
        }
    }

    @Override
    public List<TabSysMenu> listByParentMenuIdList(List<Integer> parentMenuIdList) {
        if (CollectionUtils.isEmpty(parentMenuIdList)) {
            return new ArrayList<>();
        }
        return tabSysMenuMapper.listByParentMenuIdList(parentMenuIdList);
    }

    private void updateSort(Integer sort, Integer id) {
        TabSysMenu record = new TabSysMenu();
        record.setMenuId(id);
        record.setSort(sort);
        tabSysMenuMapper.updateByPrimaryKeySelective(record);
    }
}
