package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.BusinessFrontResDTO;
import com.gic.auth.entity.TabSysBusinessFrontRes;
import com.gic.auth.service.BusinessFrontResApiService;
import com.gic.auth.service.BusinessFrontResService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

@Service("businessFrontResApiService")
public class BusinessFrontResApiServiceImpl implements BusinessFrontResApiService {
    @Autowired
    private BusinessFrontResService businessFrontResService;

    @Override
    public ServiceResponse<Integer> save(BusinessFrontResDTO dto) {
        //平台级的，没有商户ID

        if (StringUtils.isBlank(dto.getResourceName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源名称不能为空");
        }
        if (StringUtils.isBlank(dto.getResourceCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源code不能为空");
        }
        if (StringUtils.isBlank(dto.getResourceName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源名称不能为空");
        }
        if (StringUtils.isBlank(dto.getResourceDetail())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面URL/组件name不能为空");
        }
        boolean isNotRightResourceType = dto.getResourceType() == null
                || (dto.getResourceType().intValue() != 1 && dto.getResourceType().intValue() != 2);
        if (isNotRightResourceType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "类型非法");
        }
        return ServiceResponse.success(businessFrontResService.save(dto));
    }

    @Override
    public ServiceResponse<String> edit(BusinessFrontResDTO dto) {
        TabSysBusinessFrontRes record = businessFrontResService.getByResourceId(dto.getResourceId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "无该记录");
        }
        if (StringUtils.isBlank(dto.getResourceName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源名称不能为空");
        }
        if (StringUtils.isBlank(dto.getResourceCode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源code不能为空");
        }
        if (StringUtils.isBlank(dto.getResourceName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源名称不能为空");
        }
        if (StringUtils.isBlank(dto.getResourceDetail())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面URL/组件name不能为空");
        }
        boolean isNotRightResourceType = dto.getResourceType() == null
                || (dto.getResourceType().intValue() != 1 && dto.getResourceType().intValue() != 2);
        if (isNotRightResourceType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "类型非法");
        }
        businessFrontResService.edit(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> delete(Integer resourceId) {
        TabSysBusinessFrontRes record = businessFrontResService.getByResourceId(resourceId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "无该记录");
        }
        businessFrontResService.delete(resourceId);
        return ServiceResponse.success(record.getResourceName());
    }

    @Override
    public ServiceResponse<List<BusinessFrontResDTO>> listByParams(String search, Integer resourceType) {
        List<TabSysBusinessFrontRes> list = businessFrontResService.listByParam(search, resourceType);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(BusinessFrontResDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }
}
