package com.gic.auth.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.ResourceTypeEnum;
import com.gic.auth.dto.UnionEnterpriseDTO;
import com.gic.auth.dto.UnionEnterpriseResourceDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.qo.PageQO;
import com.gic.auth.web.qo.UnionEnterpriseQO;
import com.gic.auth.web.vo.GoodsResouceVO;
import com.gic.auth.web.vo.StoreResouceVO;
import com.gic.auth.web.vo.UnionEnterpriseVO;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.goods.api.dto.GoodsRightsSelectorDTO;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
public class UnionEnterpriseController {
    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private UnionEnterpriseApiService unionEnterpriseApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;
    @Autowired
    private UserApiService userApiService;

    @RequestMapping("save-union-enterprise")
    public RestResponse saveUnionEnterprise(@RequestBody UnionEnterpriseQO unionEnterpriseQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        UnionEnterpriseDTO enterpriseDTO = EntityUtil.changeEntityByJSON(UnionEnterpriseDTO.class, unionEnterpriseQO);
        enterpriseDTO.setEnterpriseId(enterpriseId);
        StoreWidgetDTO storeWidgetDTO = new StoreWidgetDTO();
        storeWidgetDTO.setAuthMode(unionEnterpriseQO.getStoreResouce().getAuthMode());
        storeWidgetDTO.setSearchParam(unionEnterpriseQO.getStoreResouce().getSearchParam());
        storeWidgetDTO.setStoreWidgetId(unionEnterpriseQO.getStoreResouce().getStoreResourceId());
        storeWidgetDTO.setWriteBackParam(unionEnterpriseQO.getStoreResouce().getWriteBackParam());
        ServiceResponse<Integer> storeWidgetResponse = this.storeWidgetApiService.saveAuthStoreWidget(storeWidgetDTO);
        GoodsRightsSelectorDTO goodsRightsSelectorDTO = EntityUtil.changeEntityByJSON(GoodsRightsSelectorDTO.class, unionEnterpriseQO.getGoodsResource());
        ServiceResponse<GoodsRightsSelectorDTO> goodsResponse = this.goodsRightsSelectorApiService.saveGoodsRightsSelector(goodsRightsSelectorDTO);
        if(storeWidgetResponse.isSuccess()){
            enterpriseDTO.setStoreResourceId(storeWidgetResponse.getResult().longValue());
        }
        if(goodsResponse.isSuccess()){
            enterpriseDTO.setGoodsResourceId(goodsResponse.getResult().getGoodsRightsSelectorId());
        }
        ServiceResponse<Integer> response = this.unionEnterpriseApiService.saveUnionEnterprise(enterpriseDTO);
        if(response.isSuccess()){
            LogUtils.createLog("联合商户创建", unionEnterpriseQO.getUnionEnterpriseName());
            return RestResponse.success(response.getResult());
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("check-auth-code")
    public RestResponse checkAuthCode(Integer authCodeId, String authCode){
        ServiceResponse response = this.authCodeApiService.validateAuthCode(authCodeId, authCode);
        this.authCodeApiService.expireAuthCode(authCodeId);
        if(response.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("union-enterprise-detail")
    public RestResponse getUnionEnterpriseDetail(Integer unionId){
        if(unionId == null){
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        ServiceResponse<UnionEnterpriseDTO> response = this.unionEnterpriseApiService.getUnionEnterpriseById(unionId);
        if(response.isSuccess()){
            UnionEnterpriseVO vo = EntityUtil.changeEntityByJSON(UnionEnterpriseVO.class, response.getResult());
            ServiceResponse<StoreWidgetDTO> storeWidget = this.storeWidgetApiService.getStoreWidget(response.getResult().getStoreResourceId().intValue());
            if(storeWidget.isSuccess()){
                if(storeWidget.getResult() != null){
                    StoreResouceVO storeResouceVO = new StoreResouceVO();
                    storeResouceVO.setStoreResourceId(response.getResult().getStoreResourceId().intValue());
                    storeResouceVO.setAuthMode(storeWidget.getResult().getAuthMode());
                    storeResouceVO.setSearchParam(storeWidget.getResult().getSearchParam());
                    storeResouceVO.setWriteBackParam(storeWidget.getResult().getWriteBackParam());
                    vo.setStoreResouce(storeResouceVO);
                }
            }
            ServiceResponse<GoodsRightsSelectorDTO> goodsResponse = this.goodsRightsSelectorApiService.getGoodsRightsSelector(response.getResult().getGoodsResourceId(), UserDetailUtils.getUserDetail().getEnterpriseId());
            if(goodsResponse.isSuccess()){
                if(goodsResponse.getResult() != null){
                    vo.setGoodsResouce(EntityUtil.changeEntityByJSON(GoodsResouceVO.class, goodsResponse.getResult()));
                }
            }
            return RestResponse.success(vo);
        }
        return EnterpriseRestResponse.failure(response);
    }

    @RequestMapping("del-union-enterprise-resource")
    public RestResponse delUnionEnterprise(Long resourceId, Integer resourceType){
        if(resourceId == null){
            return EnterpriseRestResponse.failure(ErrorCode.MISS_PARAMETER);
        }
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Integer> response = this.unionEnterpriseApiService.delUnionEnterpriseSource(resourceId, enterpriseId);
        if(response.getResult()>0){
            ResourceTypeEnum[] values = ResourceTypeEnum.values();
            for(ResourceTypeEnum typeEnum : values){
                if(resourceType == typeEnum.getCode()){
                    LogUtils.createLog(typeEnum.getMessage(), "资源id："+resourceId);
                }
            }
            return RestResponse.success();
        }
        return RestResponse.failure(ErrorCode.NOTEXISTS.getCode(), "资源不存在");
    }

    @RequestMapping("re-authorization-resource")
    public RestResponse reAuthorizationResouce(Long resourceId, int resourceType){
        this.unionEnterpriseApiService.reAuthorizationResource(resourceId, resourceType);
        return RestResponse.success();
    }

    @RequestMapping("page-union-enterprise")
    public RestResponse pageUnionEnterprise(String search, Integer authorizationStatus, PageQO pageQO){
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        ServiceResponse<Page<UnionEnterpriseDTO>> pageServiceResponse = this.unionEnterpriseApiService.
                pageUnionEnterprise(search, enterpriseId, pageQO.getCurrentPage(), pageQO.getPageSize(), authorizationStatus);
        Page<UnionEnterpriseDTO> result = pageServiceResponse.getResult();
        if(result != null){
            List<UnionEnterpriseDTO> list = result.getResult();
            if(CollectionUtils.isNotEmpty(list)){
                for(UnionEnterpriseDTO dto : list){
                    ServiceResponse<List<UnionEnterpriseResourceDTO>> response = this.unionEnterpriseApiService.listResourceByUnionId(dto.getUnionId());
                    dto.setResourceList(response.getResult());
                    dto.setUnionCompanyName(UserDetailUtils.getUserDetail().getEnterpriseInfo().getCompanyName());
                    ServiceResponse<UserDTO> userResponse = this.userApiService.getUserByEnterpriseId(enterpriseId);
                    if(userResponse.isSuccess()){
                        dto.setUnionManagerName(userResponse.getResult().getUserName());
                        dto.setUnionManagerPhone(userResponse.getResult().getPhoneNumber());
                    }
                }
            }
            return RestResponse.success(result);
        }
        return RestResponse.success();
    }
}
